//****************************************************************************
//  WS2811.h
//  WS2811 LED Strip Driver with Patterns
//  by RCI
//  Thanks to Patrick Cantwell for the cool WS2811 Frame Routine!
//  Modified with RAO ADC_Read(0) for speed
//
//****************************************************************************
// Color Order for WS2811 LED is GRB
// User may need to Change orientation
//
// Color Definitions     Order G, R, B
#define        red            0,255,0
#define        green          255,0,0
#define        blue           0,0,255

#define        black          0,0,0
#define        grey           40,40,40
#define        white          255,255,255

#define        yellow         255,255,0
#define        cyan           255,0,255
#define        magenta        0,255,255
#define        gold           160,255,0
#define        orange         20,255,0

#define        olive          220,100,47
#define        turq           245,40,100
#define        lime           254,64,20
#define        skyblue        206,135,255
#define        purple         0,75,130
#define        pink           30,255,12
#define        crimson        3,220,3

#define  testbit(var, bit)  ((var) & (1 <<(bit)))
#define  setbit(var, bit)   ((var) |= (1 << (bit)))
#define  clrbit(var, bit)   ((var) &= ~(1 << (bit)))
#define  ws2811_zero 0b10000000        // was 10000000
#define  ws2811_one  0b11110000        // was 11110000
#define  tail 10                       // Sets Tail Length for Comet Routine
//****************************************************************************
//Variable Declarations
//****************************************************************************
char MAX_LED_COUNT;
signed long led_strip_colors[256];
int i, j;
char txt[7];
char row, col, Disp_Enable;
char AtoD_Ch;
unsigned char last_event;
signed int v;
unsigned char brightness = 1;
unsigned int speede;
//****************************************************************************
void WS2811_AtoD_Display(char row_temp, char col_temp, int val, char disp_ena)
{
  AtoD_Ch = val;
  Disp_Enable = disp_ena;
  if(Disp_Enable == 0)
  {
  speede = 10;
  }
  row = row_temp;
  col = col_temp;
  return;
}
//****************************************************************************
void WS2811_frame()
{
  unsigned int i;
  unsigned long this_led;
  unsigned int loop;

  if(Disp_Enable == 1)
  {
  speede = ADC_Read(AtoD_Ch);
  InttoStr(speede, txt);
  LCD_Out(row, col, txt);
  LCD_Out(row, col + 6, " ms");
  }

  for(i = 0; i < MAX_LED_COUNT; i++)
  {
  this_led = led_strip_colors[i];
  for(loop = 0; loop < 24; loop++)
      {
      if(testbit(this_led, 23))
          {          // test fixed bit 23
          spi1_write(ws2811_one);
          }
      else
          {
          spi1_write(ws2811_zero);
          }
      this_led *= 2;  // march all the bits one position to the left so
      }
  }
  delay_us(75);
  vdelay_ms(speede);
}
//****************************************************************************
void WS2811_fixed(int frame_speed)
{
  unsigned int i;
  unsigned long this_led;
  unsigned int loop;

  for(i = 0; i < MAX_LED_COUNT; i++)
  {
  this_led = led_strip_colors[i];
  for(loop = 0; loop < 24; loop++)
      {
      if(testbit(this_led, 23))
          {          // test fixed bit 23
          spi1_write(ws2811_one);
          }
      else
          {
          spi1_write(ws2811_zero);
          }
      this_led *= 2;  // march all the bits one position to the left so
      }
  }
  delay_us(75);
  vdelay_ms(frame_speed);
}

//****************************************************************************
void WS2811_LEDCount(char vari)
{
  MAX_LED_COUNT = vari + 6;
  return;
}
//****************************************************************************
void Clear_Strip()
{
char pos;
    // Clear Strip
    for(pos = 0; pos < MAX_LED_COUNT; pos++)
       {
       led_strip_colors[pos] = 0x000000;
       }
       WS2811_frame();
}
//*********************************************************************************
// Convert separate R,G,B into packed 32-bit RGB color.
// Packed format is always RGB, regardless of LED strand color order.
unsigned long Strip_Color(char g, char r, char b)
   {
   return ((unsigned long)g << 16) | ((unsigned long)r <<  8) | b;
   }
//****************************************************************************
Set_Color(unsigned long rred, unsigned int ggreen, char bblue,
               char position)
{
   unsigned long temp, temp1;

   temp = 0;
   temp1 = 0;
   temp1 = (rred / brightness) << 16;
   temp = temp + temp1;
   temp1 = 0;
   temp1 = (ggreen / brightness) << 8;
   temp = temp + temp1;
   temp1 = 0;
   temp1 = (bblue / brightness);
   temp = temp + temp1;
   led_strip_colors[position] = temp;
}
//*********************************************************************************
// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
unsigned long Wheel(char WheelPos)
{
  if(WheelPos < 85)
      {
      return Strip_Color(WheelPos * 3, 255 - WheelPos * 3, 0);
      }
  else if(WheelPos < 170)
      {
      WheelPos -= 85;
      return Strip_Color(255 - WheelPos * 3, 0, WheelPos * 3);
      }
  else
      {
      WheelPos -= 170;
      return Strip_Color(0, WheelPos * 3, 255 - WheelPos * 3);
      }
}
//****************************************************************************
void RandomColor(int loops, int frame_speed)
{
unsigned char a, b, c, d;

for (j = 0; j < loops; j++)
    {
    srand(j);
    a = rand() % 255;
    b = rand() % 255;
    c = rand() % 255;
    d = rand() % MAX_LED_COUNT;
//    d = (rand() * (MAX_LED_COUNT + 1) & 127);
    Set_Color(a, b, c, d);
    WS2811_fixed(frame_speed);
    }
}
//****************************************************************************
void ChaseRandom(int loops)
{
  unsigned int x;
  unsigned long new_color = 0;
for (j = 0; j < loops; j++)
  {
  //First, shuffle all the current colors down one spot on the strip
  for(x = (MAX_LED_COUNT - 1); x > 0; x--)
    {
    led_strip_colors[x] = led_strip_colors[x - 1];
    }
  //Now generate a new RGB color
  for(x = 0; x < 3; x++)
    {
    new_color <<= 8;
    new_color |= rand();
    }
  led_strip_colors[0] = new_color; //Add the new random color to the strip
  WS2811_frame();
  }
}
//****************************************************************************
void Fade_Strip(char rrednext, char ggreennext, char bbluenext,
                char rredlast, char ggreenlast, char bbluelast,
                int pos)
{
// ***Declare Temp Variables****************************
   float rlast, glast, blast, rcurr, gcurr, bcurr, rcalc, gcalc, bcalc;
   float MaxTemp, MaxStep, rtemp, gtemp, btemp;
   unsigned long temp, temp1;
   unsigned char goFlag, nnn, count;
   unsigned long rred;
   unsigned int ggreen;
   unsigned char bblue;
   
   count = 0;
   goFlag = 1;
   while(goFlag == 1)
      {
      rcurr =  rredlast;
      gcurr =  ggreenlast;
      bcurr =  bbluelast;
      rlast =  rrednext;
      glast =  ggreennext;
      blast =  bbluenext;
      MaxTemp = max((abs(rcurr - rlast)), (abs(gcurr - glast)));
      MaxStep = max((abs(bcurr - blast)), MaxTemp);
      rcalc = rlast;
      gcalc = glast;
      bcalc = blast;
     //*******************************************************
     // Calculate proportional fade rates
      for (nnn = 0; nnn < MaxStep; nnn++)
        {
        rtemp = (rcurr - rlast) / MaxStep;
        gtemp = (gcurr - glast) / MaxStep;
        btemp = (bcurr - blast) / MaxStep;
        rcalc = rcalc + rtemp;
        gcalc = gcalc + gtemp;
        bcalc = bcalc + btemp;
        rred = rcalc;
        ggreen = gcalc;
        bblue = bcalc;
     //*******************************************************
     // Construct 24bit word
        temp = 0;
        temp1 = 0;
        temp1 = rred << 16;
        temp = temp + temp1;
        temp1 = 0;
        temp1 = ggreen << 8;
        temp = temp + temp1;
        temp1 = 0;
        temp1 = bblue;
        temp = temp + temp1;

     //*******************************************************
     // Populate Faded Colors
     if(pos == 0)
        {
        for (count = 0; count < MAX_LED_COUNT + 1; ++count)
            {
            led_strip_colors[count] = temp;
            }
        }
     else
        {
        led_strip_colors[pos] = temp;
        }
      if(last_event == 1)
      {
      last_event = 0;
      return;
      }
     //*******************************************************
     // Delay and Display Frame
        WS2811_frame();
        } // End For MaxStep
        goFlag = 0;
    } // End While
}
//****************************************************************************
void Solid_Strip(char r1, char g1, char b1)
{
char pos;

    for(pos = 0; pos < MAX_LED_COUNT; pos++)
       {
       if(last_event == 1)
       {
       last_event = 0;
       return;
       }
       Set_Color(r1, g1, b1, pos);
       }
       WS2811_frame();
}
//****************************************************************************
void InsertColor(char r1, char g1, char b1)
{
  unsigned int x;
  unsigned long new_color = 0;

  for(x = (MAX_LED_COUNT - 1); x > 0; x--)
    {
    led_strip_colors[x] = led_strip_colors[x - 1];
    }
  Set_Color(r1, g1, b1, 0);
    if(last_event == 1)
    {
    last_event = 0;
    return;
    }
  WS2811_frame();

}
//****************************************************************************
void FillLeft(char r1, char g1, char b1)
{
  unsigned char count;
    for(count = MAX_LED_COUNT; count > 0; count--)
      {
      if(last_event == 1)
      {
      last_event = 0;
      return;
      }
      Set_Color(r1, g1, b1, count - 1);
      WS2811_frame();
      }
}
//****************************************************************************
void FillRight(char r1, char g1, char b1)
{
  char count;
    for(count = 0; count < MAX_LED_COUNT; count++)
      {
      if(last_event == 1)
      {
      last_event = 0;
      return;
      }
      Set_Color(r1, g1, b1, count);
      WS2811_frame();
      }
}
//****************************************************************************
//  Fill Right Any Position
void Fill_Any_PosR
(char r1, char g1, char b1, char r2, char g2, char b2,
                       char r3, char g3, char b3, char start, char end)
{
char pos;
   for(pos = start; pos < end; pos++)
      {
      if(last_event == 1)
      {
      last_event = 0;
      return;
      }
      if((pos%3) == 0) {Set_Color(r3, g3, b3, pos);}
      if((pos%3) == 1) {Set_Color(r2, g2, b2, pos);}
      if((pos%3) == 2) {Set_Color(r1, g1, b1, pos);}

      WS2811_frame();
      }
}// End Fill_Right3
//****************************************************************************
//  Fill Left Any Position
void Fill_Any_PosL(char r1, char g1, char b1, char r2, char g2, char b2,
                       char r3, char g3, char b3, char start, char end)
{
char pos;
   for(pos = start; pos > end; pos--)
      {
      if(last_event == 1)
      {
      last_event = 0;
      return;
      }
      if((pos%3) == 0) {Set_Color(r1, g1, b1, pos - 1);}
      if((pos%3) == 1) {Set_Color(r2, g2, b2, pos - 1);}
      if((pos%3) == 2) {Set_Color(r3, g3, b3, pos - 1);}

      WS2811_frame();
      }
}// End Fill_Left3
//****************************************************************************
void Chase_3Color_Left(char r1, char g1, char b1, char r2, char g2, char b2,
                       char r3, char g3, char b3, int repeat)
{
char i, strt, pos, loopframe;

      pos = MAX_LED_COUNT;
      for(loopframe = 0; loopframe < repeat; loopframe++)
         {
         // Increments Led Position every ++pos
         Set_Color(r3, g3, b3, pos + 2);
         Set_Color(r2, g2, b2, pos + 1);
         Set_Color(r1, g1, b1, pos);
         pos--;
         if (pos == MAX_LED_COUNT - 3)
            {
            pos = MAX_LED_COUNT;
            }

         for(strt = 0; strt < MAX_LED_COUNT; strt++)
            {
            led_strip_colors[strt] = led_strip_colors[strt + 1];
            }
            WS2811_frame();
         }// End for loopframe

}// End Chase_3Color_Left
//****************************************************************************
void Chase_3Color_Right(char r1, char g1, char b1, char r2, char g2, char b2,
                       char r3, char g3, char b3, int repeat)
{
char i, loopframe;
char pos;
signed int strt;

      pos = 0;
      // Increments loopframe MAX_LED_COUNT counts including zero
      for(loopframe = 0; loopframe < repeat; loopframe++)
         {
         if(last_event == 1)
         {
         last_event = 0;
         return;
         }
         // Increments Led Position every ++pos
         Set_Color(r1, g1, b1, pos - 0);
         Set_Color(r2, g2, b2, pos - 1);
         Set_Color(r3, g3, b3, pos - 2);
         pos++;
         if(pos == 3)
            {
            pos = 0;
            }
         // Counts down from MAX_LED_COUNT to 0
         for(strt = MAX_LED_COUNT; strt >= 0; strt--)
            {
            led_strip_colors[strt + 1] = led_strip_colors[strt];
            }
            WS2811_frame();
         }// End for loopframe

}// End Chase_Right3
//****************************************************************************
void MovLeft_Chase_and_Clear(char position)
{
char pos, loopframe;
      for(loopframe = 0; loopframe < position; loopframe++)
         {
         if(last_event == 1)
         {
         last_event = 0;
         return;
         }
         for(pos = 0; pos < MAX_LED_COUNT; pos++)
            {
            led_strip_colors[pos] = led_strip_colors[pos + 1];
            }
         WS2811_frame();
         led_strip_colors[MAX_LED_COUNT] = 0x000000;
         }
}
//****************************************************************************
void MovRight_Chase_and_Clear(char position)
{
char pos, loopframe;
      for(loopframe = 0; loopframe < position; loopframe++)
         {
         if(last_event == 1)
         {
         last_event = 0;
         return;
         }
         for(pos = (MAX_LED_COUNT - 1); pos > 0; pos--)
            {
            led_strip_colors[pos] = led_strip_colors[pos - 1];
            }
         WS2811_frame();
         led_strip_colors[0] = 0x000000;
         }
}
//****************************************************************************
void Fill_Dot (char r1, char g1, char b1, char pos)
{
      Set_Color(r1, g1, b1, pos);
      WS2811_frame();

}// End Fill_Right3
//****************************************************************************
void Fill_Dot_PosR (char r1, char g1, char b1, char start, char end)
{
unsigned char pos;
   for(pos = start; pos <= end; pos++)
      {
      Set_Color(r1, g1, b1, pos);
      }
      WS2811_frame();

}// End Fill_Right3
//****************************************************************************
void Fill_Dot_PosL(char r1, char g1, char b1, char start, char end)
{
char pos;
   for(pos = start; pos >= end; pos--)
      {
      Set_Color(r1, g1, b1, pos - 1);
      }
      WS2811_frame();

}// End Fill_Left3
//*********************************************************************************
// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycleLeft(unsigned char cycles, int frame_speed)
{
  for(j = 0; j < 256 * cycles; j++)
      {
      for(i = 0; i < MAX_LED_COUNT; i++)
          {
          led_strip_colors[i] = Wheel(((i * 256 / MAX_LED_COUNT) + j) & 255);
          }
          WS2811_frame();
          //WS2811_fixed(frame_speed);
      }
}
//*********************************************************************************
// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycleRight(unsigned char cycles, int frame_speed)
{
  for(j = 256 * cycles; j--;)
      {
      for(i = 0; i < MAX_LED_COUNT; i++)
          {
          led_strip_colors[i] = Wheel(((i * 256 / MAX_LED_COUNT) + j) & 255);
          }
          WS2811_frame();
          //WS2811_fixed(frame_speed);
      }
}
//***************************************************************************
void CometLeft(unsigned char rred, unsigned char ggreen, unsigned char bblue)
{

  for(v = 0; v < MAX_LED_COUNT + tail - 1; v++)
      {
      Set_Color(rred, ggreen, bblue, v);
      Set_Color(rred / 2, ggreen / 2, bblue / 2, v - 1);
      Set_Color(rred / 4, ggreen / 4, bblue / 4, v - 2);
      Set_Color(rred / 8, ggreen / 8, bblue / 8, v - 3);
      Set_Color(rred / 16, ggreen / 16, bblue / 16, v - 4);
      Set_Color(rred / 32, ggreen / 32, bblue / 32, v - 5);
      Set_Color(rred / 64, ggreen / 64, bblue / 64, v - 6);
      Set_Color(rred / 128, ggreen / 128, bblue / 128, v - 7);
      Set_Color(0, 0, 0, v - 8);
      WS2811_frame();
      }
      return;
}
//*****************************************************************************
void CometRight(unsigned char rred, unsigned char ggreen, unsigned char bblue)
{
  for(v = MAX_LED_COUNT; v--;)
      {
      Set_Color(rred, ggreen, bblue, v - 9);
      Set_Color(rred / 2, ggreen / 2, bblue / 2, v - 8);
      Set_Color(rred / 4, ggreen / 4, bblue / 4, v - 7);
      Set_Color(rred / 8, ggreen / 8, bblue / 8, v - 6);
      Set_Color(rred / 16, ggreen / 16, bblue / 16, v - 5);
      Set_Color(rred / 32, ggreen / 32, bblue / 32, v - 4);
      Set_Color(rred / 64, ggreen / 64, bblue / 64, v - 3);
      Set_Color(rred / 128, ggreen / 128, bblue / 128, v - 2 );
      Set_Color(0, 0, 0, v - 1);
      WS2811_frame();
      }
}
//*****************************************************************************